import json
import sys

from insights.core import dr
from insights.core.evaluators import SingleEvaluator as Evaluator, get_simple_module_name
from insights.formats import EvaluatorFormatterAdapter, get_response_of_types, render


class JsonFormat(Evaluator):
    def __init__(self, broker=None, missing=False, render_content=False, show_rules=None, stream=sys.stdout):
        super(JsonFormat, self).__init__(broker, stream=stream)
        self.missing = missing
        self.render_content = render_content
        self.show_rules = [] if show_rules is None else show_rules

    def handle_result(self, plugin, r):
        type_ = r["type"]

        if type_ == "skip":
            self.rule_skips.append(r)
        elif type_ == "metadata":
            self.append_metadata(r)
        elif type_ == "metadata_key":
            self.metadata_keys[r.get_key()] = r["value"]
        else:
            comp = dr.get_name(plugin)
            key = r.get_key()
            response_id = "%s_id" % r.response_type
            result = {
                response_id: "{0}|{1}".format(get_simple_module_name(plugin), key),
                "component": comp,
                "type": type_,
                "key": key,
                "details": r,
                "tags": list(dr.get_tags(plugin)),
                "links": dr.get_delegate(plugin).links or {}
            }
            if self.render_content:
                result.update({"rendered_content": render(comp, r)})

            self.results[type_].append(self.format_result(result))

    def postprocess(self):
        response = get_response_of_types(self.get_response(), self.missing, self.show_rules)
        json.dump(response, self.stream)


class JsonFormatterAdapter(EvaluatorFormatterAdapter):
    Impl = JsonFormat



from insights.parsers.jboss_standalone_main_conf import JbossStandaloneConf
from insights.tests import context_wrap
from insights.parsers import jboss_standalone_main_conf
import doctest

JBOSS_STANDALONE_CONFIG = """
<?xml version='1.0' encoding='UTF-8'?>

<server xmlns="urn:jboss:domain:1.7">
    <management>
        <security-realms>
            <security-realm name="ManagementRealm">
                <authentication>
                    <local default-user="$local" skip-group-loading="true"/>
                    <properties path="mgmt-users.properties" relative-to="jboss.server.config.dir"/>
                </authentication>
                <authorization map-groups-to-roles="false">
                    <properties path="mgmt-groups.properties" relative-to="jboss.server.config.dir"/>
                </authorization>
            </security-realm>
            <security-realm name="ApplicationRealm">
                <authentication>
                    <local default-user="$local" allowed-users="*" skip-group-loading="true"/>
                    <properties path="application-users.properties" relative-to="jboss.server.config.dir"/>
                </authentication>
                <authorization>
                    <properties path="application-roles.properties" relative-to="jboss.server.config.dir"/>
                </authorization>
            </security-realm>
        </security-realms>
        <audit-log>
            <formatters>
                <json-formatter name="json-formatter"/>
            </formatters>
            <handlers>
                <file-handler name="file" formatter="json-formatter" path="audit-log.log" relative-to="jboss.server.data.dir"/>
            </handlers>
            <logger log-boot="true" log-read-only="false" enabled="false">
                <handlers>
                    <handler name="file"/>
                </handlers>
            </logger>
        </audit-log>
        <management-interfaces>
            <native-interface security-realm="ManagementRealm">
                <socket-binding native="management-native"/>
            </native-interface>
            <http-interface security-realm="ManagementRealm">
                <socket-binding http="management-http"/>
            </http-interface>
        </management-interfaces>
        <access-control provider="simple">
            <role-mapping>
                <role name="SuperUser">
                    <include>
                        <user name="$local"/>
                    </include>
                </role>
            </role-mapping>
        </access-control>
    </management>
</server>
"""


def test_jboss_standalone_conf():
    jboss_standalone_conf = JbossStandaloneConf(
        context_wrap(JBOSS_STANDALONE_CONFIG, path="/root/jboss/jboss-eap-6.4/standalone/configuration/standalone.xml"))
    assert jboss_standalone_conf is not None
    assert jboss_standalone_conf.file_path == "/root/jboss/jboss-eap-6.4/standalone/configuration/standalone.xml"
    assert jboss_standalone_conf.get_elements(
        ".//management/security-realms/security-realm/authentication/properties")[0].get(
        "relative-to") == 'jboss.server.config.dir'


def test_jboss_standalone_conf_doc_examples():
    env = {
        'JbossStandaloneConf': JbossStandaloneConf,
        'jboss_main_config': JbossStandaloneConf(context_wrap(JBOSS_STANDALONE_CONFIG,
                                                              path='/root/jboss/jboss-eap-6.4/standalone/configuration/standalone.xml'))
    }
    failed, total = doctest.testmod(jboss_standalone_main_conf, globs=env)
    assert failed == 0



from insights.parsr.examples.nginx_conf import loads


NGINX_CONF = r"""
user       www www;  ## Default: nobody
worker_processes  5;  ## Default: 1
error_log  logs/error.log;
pid        logs/nginx.pid;
worker_rlimit_nofile 8192;

events {
  worker_connections  4096;  ## Default: 1024
}

http {
  include    conf/mime.types;
  include    /etc/nginx/proxy.conf;
  include    /etc/nginx/fastcgi.conf;
  index    index.html index.htm index.php;

  default_type application/octet-stream;
  log_format   main '$remote_addr - $remote_user [$time_local]  $status '
    '"$request" $body_bytes_sent "$http_referer" '
    '"$http_user_agent" "$http_x_forwarded_for"';
  access_log   logs/access.log  main;
  sendfile     on;
  tcp_nopush   on;
  server_names_hash_bucket_size 128; # this seems to be required for some vhosts

  server { # php/fastcgi
    listen       80;
    server_name  domain1.com www.domain1.com;
    access_log   logs/domain1.access.log  main;
    root         html;

    location ~ \.php$ {
      fastcgi_pass   127.0.0.1:1025;
    }
  }

  server { # simple reverse-proxy
    listen       80;
    server_name  domain2.com www.domain2.com;
    access_log   logs/domain2.access.log  main;

    # serve static files
    location ~ ^/(images|javascript|js|css|flash|media|static)/  {
      root    /var/www/virtual/big.server.com/htdocs;
      expires 30d;
    }

    # pass requests for dynamic content to rails/turbogears/zope, et al
    location / {
      proxy_pass      http://127.0.0.1:8080;
    }
  }

  upstream big_server_com {
    server 127.0.0.3:8000 weight=5;
    server 127.0.0.3:8001 weight=5;
    server 192.168.0.1:8000;
    server 192.168.0.1:8001;
  }

  server { # simple load balancing
    listen          80;
    server_name     big.server.com;
    access_log      logs/big.server.access.log main;

    location / {
      proxy_pass      http://big_server_com;
    }
  }
}"""


PROXY_CONF = """
proxy_redirect          off;
proxy_set_header        Host            $host;
proxy_set_header        X-Real-IP       $remote_addr;
proxy_set_header        X-Forwarded-For $proxy_add_x_forwarded_for;
client_max_body_size    10m;
client_body_buffer_size 128k;
proxy_connect_timeout   90;
proxy_send_timeout      90;
proxy_read_timeout      90;
proxy_buffers           32 4k;
"""


FAST_CGI = """
fastcgi_param  SCRIPT_FILENAME    $document_root$fastcgi_script_name;
fastcgi_param  QUERY_STRING       $query_string;
fastcgi_param  REQUEST_METHOD     $request_method;
fastcgi_param  CONTENT_TYPE       $content_type;
fastcgi_param  CONTENT_LENGTH     $content_length;
fastcgi_param  SCRIPT_NAME        $fastcgi_script_name;
fastcgi_param  REQUEST_URI        $request_uri;
fastcgi_param  DOCUMENT_URI       $document_uri;
fastcgi_param  DOCUMENT_ROOT      $document_root;
fastcgi_param  SERVER_PROTOCOL    $server_protocol;
fastcgi_param  GATEWAY_INTERFACE  CGI/1.1;
fastcgi_param  SERVER_SOFTWARE    nginx/$nginx_version;
fastcgi_param  REMOTE_ADDR        $remote_addr;
fastcgi_param  REMOTE_PORT        $remote_port;
fastcgi_param  SERVER_ADDR        $server_addr;
fastcgi_param  SERVER_PORT        $server_port;
fastcgi_param  SERVER_NAME        $server_name;

fastcgi_index  index.php;

fastcgi_param  REDIRECT_STATUS    200;
"""


MIME_TYPES = """
types {
  text/html                             html htm shtml;
  text/css                              css;
  text/xml                              xml rss;
  image/gif                             gif;
  image/jpeg                            jpeg jpg;
  application/x-javascript              js;
  text/plain                            txt;
  text/x-component                      htc;
  text/mathml                           mml;
  image/png                             png;
  image/x-icon                          ico;
  image/x-jng                           jng;
  image/vnd.wap.wbmp                    wbmp;
  application/java-archive              jar war ear;
  application/mac-binhex40              hqx;
  application/pdf                       pdf;
  application/x-cocoa                   cco;
  application/x-java-archive-diff       jardiff;
  application/x-java-jnlp-file          jnlp;
  application/x-makeself                run;
  application/x-perl                    pl pm;
  application/x-pilot                   prc pdb;
  application/x-rar-compressed          rar;
  application/x-redhat-package-manager  rpm;
  application/x-sea                     sea;
  application/x-shockwave-flash         swf;
  application/x-stuffit                 sit;
  application/x-tcl                     tcl tk;
  application/x-x509-ca-cert            der pem crt;
  application/x-xpinstall               xpi;
  application/zip                       zip;
  application/octet-stream              deb;
  application/octet-stream              bin exe dll;
  application/octet-stream              dmg;
  application/octet-stream              eot;
  application/octet-stream              iso img;
  application/octet-stream              msi msp msm;
  audio/mpeg                            mp3;
  audio/x-realaudio                     ra;
  video/mpeg                            mpeg mpg;
  video/quicktime                       mov;
  video/x-flv                           flv;
  video/x-msvideo                       avi;
  video/x-ms-wmv                        wmv;
  video/x-ms-asf                        asx asf;
  video/x-mng                           mng;
}
"""


def test_nginx_conf():
    assert len(loads(NGINX_CONF)) == 7


def test_nginx_proxy():
    assert len(loads(PROXY_CONF)) == 10


def test_nginx_fastcgi():
    assert len(loads(FAST_CGI)) == 19


def test_nginx_mime_types():
    res = loads(MIME_TYPES)
    assert len(res) == 1
    assert len(res[0]) == 46



import pytest
import doctest

from insights.core.exceptions import ParseException
from insights.parsers import parted
from insights.parsers.parted import PartedDevice, PartedL
from insights.tests import context_wrap

PARTED_DATA = """
Model: Virtio Block Device (virtblk)
Disk /dev/vda: 9664MB
Sector size (logical/physical): 512B/512B
Partition Table: msdos
Disk Flags:

Number  Start   End     Size    Type     File system  Flags
 1      1049kB  525MB   524MB   primary  xfs          boot
 2      525MB   9664MB  9138MB  primary               lvm
""".strip()

PARTED_DATA_NO_SECTOR_SPLIT = """
Model: Virtio Block Device (virtblk)
Disk /dev/vda: 9664MB
Sector size (logical/physical): 512B
Partition Table: msdos
Disk Flags:

Number  Start   End     Size    Type     File system  Flags
 1      1049kB  525MB   524MB   primary  xfs          boot
 2      525MB   9664MB  9138MB  primary               lvm
"""

PARTED_DATA_2 = """
Model: IBM 2107900 (scsi)
Disk /dev/sdet: 2147MB
Sector size (logical/physical): 512B/512B
Partition Table: msdos

Number  Start   End     Size    Type     File system  Flags
 1      32.3kB  2580kB  2548kB  primary
""".strip()

PARTED_DATA_3 = """
Model: DELL PERC H710 (scsi)
Disk /dev/sda: 292GB
Sector size (logical/physical): 512B/512B
Partition Table: msdos

Number  Start   End     Size    Type      File system  Flags
 1      32.3kB  526MB   526MB   primary   ext3         boot
 2      526MB   9114MB  8587MB  primary   linux-swap
 3      9114MB  12.3GB  3224MB  primary   ext3
 4      12.3GB  292GB   280GB   extended
 5      12.3GB  254GB   241GB   logical   ext3
 6      254GB   281GB   26.8GB  logical   ext3
 7      281GB   285GB   4294MB  logical   ext3
 8      285GB   288GB   3224MB  logical   ext3
 9      288GB   290GB   2147MB  logical   ext3
10      290GB   292GB   2147MB  logical   ext3
""".strip()

PARTED_DATA_4 = """
Model: DELL PERC H710 (scsi)
Disk /dev/sda: 2399GB
Sector size (logical/physical): 512B/512B
Partition Table: gpt
Disk Flags: pmbr_boot

Number  Start   End     Size    File system  Name                  Flags
 1      1049kB  525MB   524MB   fat32        EFI system partition  boot
 2      525MB   567MB   41.9MB  fat32        Basic data partition  hidden
 3      567MB   2715MB  2147MB  fat32        Basic data partition
 4      2715MB  2716MB  1049kB                                     bios_grub
 5      2716MB  3240MB  524MB   xfs
 6      3240MB  2077GB  2074GB                                     lvm


Model: Linux device-mapper (linear) (dm)
Disk /dev/mapper/rhel_home: 191GB
Sector size (logical/physical): 512B/512B
Partition Table: loop
Disk Flags:

Number  Start  End    Size   File system  Flags
 1      0.00B  191GB  191GB  xfs


Model: Linux device-mapper (linear) (dm)
Disk /dev/mapper/rhel_iso: 161GB
Sector size (logical/physical): 512B/512B
Partition Table: loop
Disk Flags:

Number  Start  End    Size   File system  Flags
 1      0.00B  161GB  161GB  xfs


Model: Linux device-mapper (linear) (dm)
Disk /dev/mapper/rhel_vms: 1611GB
Sector size (logical/physical): 512B/512B
Partition Table: loop
Disk Flags:

Number  Start  End     Size    File system  Flags
 1      0.00B  1611GB  1611GB  xfs


Model: Linux device-mapper (linear) (dm)
Disk /dev/mapper/rhel_lvtest: 4194kB
Sector size (logical/physical): 512B/512B
Partition Table: loop
Disk Flags:

Number  Start  End     Size    File system  Flags
 1      0.00B  4194kB  4194kB  ext4


Model: Linux device-mapper (linear) (dm)
Disk /dev/mapper/rhel_root: 107GB
Sector size (logical/physical): 512B/512B
Partition Table: loop
Disk Flags:

Number  Start  End    Size   File system  Flags
 1      0.00B  107GB  107GB  xfs


Model: Linux device-mapper (linear) (dm)
Disk /dev/mapper/rhel_swap: 4295MB
Sector size (logical/physical): 512B/512B
Partition Table: loop
Disk Flags:

Number  Start  End     Size    File system     Flags
 1      0.00B  4295MB  4295MB  linux-swap(v1)
""".strip()

PARTED_DATA_5 = """
Model: VMware Virtual disk (scsi)
Disk /dev/sda: 94371840s
Sector size (logical/physical): 512B/512B
Partition Table: gpt
Disk Flags:

Number  Start     End        Size       File system  Name                  Flags
 1      2048s     2099199s   2097152s   fat32        EFI System Partition  boot, esp
 2      2099200s  4196351s   2097152s   xfs
 3      4196352s  83904511s  79708160s                                     lvm


Error: /dev/sdb: unrecognised disk label
Model: VMware Virtual disk (scsi)
Disk /dev/sdb: 75497472s
Sector size (logical/physical): 512B/512B
Partition Table: unknown
Disk Flags:


Model: VMware Virtual disk (scsi)
Disk /dev/sdc: 15032385536s
Sector size (logical/physical): 512B/512B
Partition Table: msdos
Disk Flags:

Number  Start  End          Size         Type     File system  Flags
 1      2048s  4294967294s  4294965247s  primary  xfs


Model: VMware Virtual disk (scsi)
Disk /dev/sdd: 629145600s
Sector size (logical/physical): 512B/512B
Partition Table: msdos
Disk Flags:

Number  Start  End         Size        Type     File system  Flags
 1      2048s  629145599s  629143552s  primary  xfs


Error: /dev/sde: unrecognised disk label
Model: VMware Virtual disk (scsi)
Disk /dev/sde: 2097152s
Sector size (logical/physical): 512B/512B
Partition Table: unknown
Disk Flags:
""".strip()

PARTED_DATA_6 = """
Model: VMware Virtual disk (scsi)
Disk /dev/sda: 94371840s
Sector size (logical/physical): 512B/512B
Partition Table: gpt
Disk Flags:

Number  Start     End        Size       File system  Name                  Flags
 1      2048s     2099199s   2097152s   fat32        EFI System Partition  boot, esp
 2      2099200s  4196351s   2097152s   xfs
 3      4196352s  83904511s  79708160s                                     lvm
""".strip()

PARTED_DATA_7 = """
Model: Virtio Block Device (virtblk)
Disk /dev/vda: 32.2GB
Sector size (logical/physical): 512B/512B
Partition Table: msdos
Disk Flags:

Number  Start   End     Size    Type     File system  Flags
 1      1049kB  525MB   524MB   primary  xfs          boot
 2      525MB   26.5GB  25.9GB  primary


"""


def test_parted_partedl():
    context = context_wrap(PARTED_DATA)
    res = PartedL(context)
    results = res.devices_info[0]
    assert results is not None
    assert results.get('model') == 'Virtio Block Device (virtblk)'
    assert results.disk == '/dev/vda'
    assert results.get('size') == '9664MB'
    assert results.get('sector_size') == '512B/512B'
    assert results.logical_sector_size == '512B'
    assert results.physical_sector_size == '512B'
    assert results.get('partition_table') == 'msdos'
    assert results.get('disk_flags') is None
    partitions = results.partitions
    assert len(partitions) == 2
    assert partitions[0].number == '1'
    assert partitions[0].start == '1049kB'
    assert partitions[0].end == '525MB'
    assert partitions[0].size == '524MB'
    assert partitions[0].file_system == 'xfs'
    assert partitions[0].get('name') is None
    assert partitions[0].type == 'primary'
    assert partitions[0].flags == 'boot'
    assert results.boot_partition is not None
    assert results.boot_partition.number == '1'

    assert partitions[1].get('file_system') == ''
    assert partitions[1].get('flags') == 'lvm'
    assert partitions[1].get('name') is None
    assert partitions[1].get('number') == '2'
    assert partitions[1].get('start') == '525MB'
    assert partitions[1].get('end') == '9664MB'
    assert partitions[1].get('size') == '9138MB'
    assert partitions[1].get('type') == 'primary'

    context = context_wrap(PARTED_DATA_2)
    res = PartedL(context)
    results = res.devices_info[0]
    assert results is not None
    assert results.disk == '/dev/sdet'
    assert len(results.partitions) == 1

    context = context_wrap(PARTED_DATA_3)
    res = PartedL(context)
    results = res.devices_info[0]
    assert results is not None
    assert results.disk == '/dev/sda'
    assert results.logical_sector_size == '512B'
    assert results.physical_sector_size == '512B'
    assert len(results.partitions) == 10

    context = context_wrap(PARTED_DATA_4)
    res = PartedL(context)
    all_disks = [dev_info.disk for dev_info in res]
    assert all_disks == ['/dev/sda',
                         '/dev/mapper/rhel_home',
                         '/dev/mapper/rhel_iso',
                         '/dev/mapper/rhel_vms',
                         '/dev/mapper/rhel_lvtest',
                         '/dev/mapper/rhel_root',
                         '/dev/mapper/rhel_swap']

    dev_sda = res.get('/dev/sda')
    for dev_info in res:
        assert dev_info == dev_sda
        break

    dev_mapper_rhel_home = res.get('/dev/mapper/rhel_root')
    assert len(dev_mapper_rhel_home.partitions) == 1
    assert res.get('/dev/sdb') is None

    context = context_wrap(PARTED_DATA_5)
    res = PartedL(context)
    assert len([dev_info.disk for dev_info in res]) == 3

    results = res.get('/dev/sdc')
    assert results.get('size') == '15032385536s'
    assert results.get('model') == 'VMware Virtual disk (scsi)'
    assert results.get('sector_size') == '512B/512B'
    assert results.get('partition_table') == 'msdos'
    assert results.get('disk_flags') is None

    context = context_wrap(PARTED_DATA_6)
    res = PartedL(context)
    assert len([dev_info.disk for dev_info in res]) == 1

    context = context_wrap(PARTED_DATA_7)
    res = PartedL(context)
    assert len([dev_info.disk for dev_info in res]) == 1


PARTED_ERR_DATA = """
Error: /dev/dm-1: unrecognised disk label
"""

PARTED_ERR_DATA_2 = """
Model: IBM 2107900 (scsi)
Sector size (logical/physical): 512B/512B
Partition Table: msdos

Number  Start   End     Size    Type     File system  Flags
 1      32.3kB  2580kB  2548kB  primary
""".strip()

PARTED_ERR_DATA_NO_PARTITIONS = """
Model: Virtio Block Device (virtblk)
Disk /dev/vda: 9664MB
Sector size (logical/physical): 512B/512B
Partition Table: msdos
Disk Flags:

"""

PARTED_ERR_DATA_EXCEPTIONS = """
Model: Virtio Block Device (virtblk)
Disk /dev/vda: 9664MB
Sector size (logical:physical): 512B:512B
Partition Table: msdos
Disk Flags:

Number  Start   End     Size    Type     File system  Flags
"""


def test_failure_modes():
    with pytest.raises(ParseException):
        PartedDevice(PARTED_ERR_DATA.split('\n'))

    context = context_wrap(PARTED_ERR_DATA)
    res = PartedL(context)
    assert len(res.devices_info) == 0

    with pytest.raises(ParseException):
        PartedDevice(PARTED_ERR_DATA_2.split('\n'))

    context = context_wrap(PARTED_ERR_DATA_2)
    res = PartedL(context)
    assert len(res.devices_info) == 0

    # Test failure to find a slash in sector size
    res = PartedL(context_wrap(PARTED_DATA_NO_SECTOR_SPLIT))
    results = res.devices_info[0]
    assert results is not None
    assert results.disk == '/dev/vda'
    assert results._sector_size is None
    assert results.logical_sector_size is None
    assert results.physical_sector_size is None

    # Object should be complete but has no data
    res = PartedL(context_wrap(PARTED_ERR_DATA_NO_PARTITIONS))
    part = res.devices_info[0]
    assert part is not None
    assert part.disk == '/dev/vda'
    assert part.data['sector_size'] == '512B/512B'
    assert part.logical_sector_size == '512B'
    assert part.physical_sector_size == '512B'
    assert part.partitions == []

    # Various other failure modes that aren't actual parse exceptions atm
    res = PartedL(context_wrap(PARTED_ERR_DATA_EXCEPTIONS))
    part = res.devices_info[0]
    assert part.disk == '/dev/vda'
    assert 'sector_size' not in part.data


PARTED_DOC_TEST_1 = """
Model: ATA TOSHIBA MG04ACA4 (scsi)
Disk /dev/sda: 4001GB
Sector size (logical/physical): 512B/512B
Partition Table: gpt
Disk Flags: pmbr_boot

Number  Start   End     Size    File system  Name  Flags
 1      1049kB  2097kB  1049kB                     bios_grub
 2      2097kB  526MB   524MB   xfs
 3      526MB   4001GB  4000GB                     lvm


Model: IBM 2107900 (scsi)
Disk /dev/sdb: 2147MB
Sector size (logical/physical): 512B/512B
Partition Table: msdos

Number  Start   End     Size    Type     File system  Flags
 1      32.3kB  2580kB  2548kB  primary
"""


def test_doc_examples():
    env = {
        'parted_l_results': PartedL(context_wrap(PARTED_DOC_TEST_1)),
    }
    failed, total = doctest.testmod(parted, globs=env)
    assert failed == 0



"""
OpenShift Get commands with configuration file
==============================================

The commands set is similar to the ``oc get`` commands. It is used to display openshift resources.
It uses the master configuration file rather than the default configuration when communicated with the client API.
It makes sure this command will only be executed on the master node of an OpenShift cluster.
This command will also not include the commands which display large size outputs.

Parsers included in this module are:

OcGetClusterRoleWithConfig - command ``oc get clusterrole --config /etc/origin/master/admin.kubeconfig``
--------------------------------------------------------------------------------------------------------

OcGetClusterRoleBindingWithConfig - command ``oc get clusterrolebinding --config /etc/origin/master/admin.kubeconfig``
----------------------------------------------------------------------------------------------------------------------
"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.parsers import parse_fixed_table
from insights.specs import Specs


@parser(Specs.oc_get_clusterrole_with_config)
class OcGetClusterRoleWithConfig(CommandParser):
    """
    Class to parse ``oc get clusterrole --config /etc/origin/master/admin.kubeconfig``

    A typical sample of the content of this file looks like::

        NAME
        admin
        asb-access
        asb-auth
        basic-user
        cluster-admin
        cluster-debugger
        cluster-reader
        cluster-status
        edit
        hawkular-metrics-admin
        management-infra-admin
        namespace-viewer
        registry-admin

    Examples:
        >>> type(oc_get_cluster_role_with_config)
        <class 'insights.parsers.openshift_get_with_config.OcGetClusterRoleWithConfig'>
        >>> oc_get_cluster_role_with_config[0]
        'admin'
    """

    def parse_content(self, content):
        if len(content) < 2 or content[0].strip() != "NAME":
            raise ParseException("invalid content: {0}".format(content) if content else 'empty file')
        self.data = list(map(lambda x: x.strip(), content[1:]))

    def __getitem__(self, item):
        return self.data[item]

    def __contains__(self, item):
        return item in self.data


@parser(Specs.oc_get_clusterrolebinding_with_config)
class OcGetClusterRoleBindingWithConfig(CommandParser):
    """
    Class to parse ``oc get clusterrolebinding --config /etc/origin/master/admin.kubeconfig``

    Attributes:
        data (list): List of dicts, each dict containing one row of the table
        rolebinding (dict): It is a dictionary in which the key is rolebinding name and the value is the role.

    A typical sample of the content of this file looks like::

        NAME                                                                  ROLE                                                                   USERS                            GROUPS                                         SERVICE ACCOUNTS                                                                   SUBJECTS
        admin                                                                 /admin                                                                                                                                                 openshift-infra/template-instance-controller
        admin-0                                                               /admin                                                                                                                                                 kube-service-catalog/default
        admin-1                                                               /admin                                                                                                                                                 openshift-ansible-service-broker/asb
        asb-access                                                            /asb-access                                                                                                                                            openshift-ansible-service-broker/asb-client
        asb-auth                                                              /asb-auth                                                                                                                                              openshift-ansible-service-broker/asb
        auth-delegator-openshift-template-service-broker                      /system:auth-delegator                                                                                                                                 openshift-template-service-broker/apiserver
        basic-users                                                           /basic-user                                                                                             system:authenticated
        cluster-admin                                                         /cluster-admin                                                                                          system:masters
        cluster-admin-0                                                       /cluster-admin                                                                                                                                         insights-scan/insights-scan

    Examples:
        >>> type(oc_get_clusterrolebinding_with_config)
        <class 'insights.parsers.openshift_get_with_config.OcGetClusterRoleBindingWithConfig'>
        >>> oc_get_clusterrolebinding_with_config.rolebinding["admin"]
        '/admin'
    """

    def parse_content(self, content):
        if len(content) < 2 or "NAME" not in content[0]:
            raise ParseException("invalid content: {0}".format(content) if content else 'empty file')

        self.data = parse_fixed_table(content, header_substitute=[('SERVICE ACCOUNTS', 'SERVICE_ACCOUNTS')])
        self.rolebinding = {}

        for rolebinding in self.data:
            rolebinding_name = rolebinding["NAME"].strip()
            rolebinding_role = rolebinding["ROLE"].strip()
            if rolebinding_name and rolebinding_role:
                self.rolebinding[rolebinding_name] = rolebinding_role



"""
PodmanList - command ``/usr/bin/podman (images|ps)``
====================================================

Parse the output of command "podman_list_images" and "podman_list_containers",
which have very similar formats with the "docker_list_images" and
"docker_list_containers",

For more details, please refer to the
`:class:insights.parsers.docker_list.DockerList`.
"""
from insights import parser
from insights.specs import Specs
from insights.parsers.docker_list import DockerListContainers, DockerListImages


@parser(Specs.podman_list_images)
class PodmanListImages(DockerListImages):
    """
    Handle the list of podman images using the
    `class:insights.parsers.docker_list.DockerListImages` parser class.

    Sample output of command ``podman images --all --no-trunc --digests``::

        REPOSITORY                           TAG                 DIGEST              IMAGE ID                                                           CREATED             SIZE
        rhel6_vsftpd                         latest              <none>              412b684338a1178f0e5ad68a5fd00df01a10a18495959398b2cf92c2033d3d02   37 minutes ago      459.5 MB
        rhel7_imagemagick                    latest              <none>              882ab98aae5394aebe91fe6d8a4297fa0387c3cfd421b2d892bddf218ac373b2   4 days ago          785.4 MB
        rhel6_nss-softokn                    latest              <none>              dd87dad2c7841a19263ae2dc96d32c501ee84a92f56aed75bb67f57efe4e48b5   5 days ago          449.7 MB

    Attributes:
        rows (list): List of row dictionaries.
        images (dict): Dictionary keyed on the value of the "REPOSITORY" fileld

    Examples:
        >>> images.rows[0]['REPOSITORY']
        'rhel6_vsftpd'
        >>> images.rows[1]['SIZE']
        '785.4 MB'
        >>> images.images['rhel6_vsftpd']['CREATED']
        '37 minutes ago'
    """
    pass


@parser(Specs.podman_list_containers)
class PodmanListContainers(DockerListContainers):
    """
    Handle the list of podman containers using the
    `class:insights.parsers.docker_list.DockerListContainers` parser class.


    Sample output of command ``podman ps --all --no-trunc --size``::

        CONTAINER ID                                                       IMAGE                                                              COMMAND                                            CREATED             STATUS                        PORTS                  NAMES               SIZE
        03e2861336a76e29155836113ff6560cb70780c32f95062642993b2b3d0fc216   rhel7_httpd                                                        "/usr/sbin/httpd -DFOREGROUND"                     45 seconds ago      Up 37 seconds                 0.0.0.0:8080->80/tcp   angry_saha          796 B (virtual 669.2 MB)
        95516ea08b565e37e2a4bca3333af40a240c368131b77276da8dec629b7fe102   bd8638c869ea40a9269d87e9af6741574562af9ee013e03ac2745fb5f59e2478   "/bin/sh -c 'yum install -y vsftpd-2.2.2-6.el6'"   51 minutes ago      Exited (137) 50 minutes ago                          tender_rosalind     4.751 MB (virtual 200.4 MB)

    Attributes:
        rows (list): List of row dictionaries.
        containers(dict): Dictionary keyed on the value of the "NAMES" field

    Examples:
        >>> containers.rows[0]['NAMES']
        'angry_saha'
        >>> containers.rows[0]['STATUS']
        'Up 37 seconds'
        >>> containers.containers['tender_rosalind']['STATUS']
        'Exited (137) 18 hours ago'
    """
    pass



"""
Limits configuration - file ``/etc/security/limits.conf`` and others
====================================================================

The ``LimitsConf`` class parser, which provides a 'rules' list that is
similar to the above but also provides a ``find_all`` method to find all the
rules that match a given set of criteria and other properties that make it
easier to use the contents of the parser.

"""

from .. import Parser, parser, get_active_lines
from insights.specs import Specs


@parser(Specs.limits_conf)
class LimitsConf(Parser):
    """
    Parse the /etc/security/limits.conf and files in /etc/security/limits.d.

    This parser reads the files and records the domain, type, item and value
    for each line.  This is available as a big list of dictionaries in the
    'items' property. Each item also contains the 'file' key, which denotes
    the file that this rule was read from.

    Lines with too few or too many parts, or with misspellings in the value
    such as 'unlimitied', are stored in a `bad_lines` list property.

    Use the ``find_all`` method to find all the the limits that apply in a
    particular situation.  Parameters are supplied as arguments - for
    example, ``find_all(domain='root')`` will find all rules that apply to
    root (including wildcard rules). These rules are sorted by domain, type
    and item, and the most specific rule is used.

    Attributes:
        bad_lines (list): all unparseable, non-comment lines found in order.
        domains (list): all the domains found, in alphabetical order.
        rules (list): a list of dictionares with the domain, type, item,
            value and file of each rule.

    Sample input::

        # Default limit for number of user's processes to prevent
        # accidental fork bombs.
        # See rhbz #432903 for reasoning.

        *          soft    nproc     4096
        root       soft    nproc     unlimited

    Examples:

        >>> limits = shared[LimitsConf][0] # At the moment this is per filename
        >>> len(limits.rules)
        2
        >>> limits.domains
        ['*', 'root']
        >>> limits.rules[0] # note value is integer
        {'domain': '*', 'type': 'soft', 'item': 'nproc', 'value': 4096, 'file': '/etc/security/limits.d/20-nproc.conf'}
        >>> limits.find_all(domain='root')
        [{'domain': '*', 'type': 'soft', 'item': 'nproc', 'value': 4096, 'file': '/etc/security/limits.d/20-nproc.conf'},
         {'domain': 'root', 'type': 'soft', 'item': 'nproc', 'value': 4096, 'file': '/etc/security/limits.d/20-nproc.conf'}]
        >>> limits.find_all(item='data')
        []
    """

    def parse_content(self, content):
        self.domains = []
        self.rules = []
        self.bad_lines = []
        for line in get_active_lines(content):
            linelist = line.split(None)
            if len(linelist) != 4:
                # Wrong number of parts = bad line, store and continue
                self.bad_lines.append(line)
                continue
            domain, typ, item, value = (linelist)
            if value == 'unlimited' or value == 'infinity':
                value = -1
            elif value.isdigit():
                value = int(value)
            else:
                # Unparseable value = bad line, store and continue
                self.bad_lines.append(line)
                continue

            # All valid values, add to data
            self.rules.append({
                'domain': domain,
                'type': typ,
                'item': item,
                'value': value,
                'file': self.file_path,
            })
            if domain not in self.domains:
                self.domains.append(domain)

        self.domains.sort()

    def _matches(self, param, ruleval, argval):
        """
        Test that a single rule value matches the given value for a given
        parameter.
        """
        if ruleval == argval:
            return True

        if param == 'domain':
            # Domains are special:
            # * Usernames are given exactly
            # * Group names are prefixed by @ and given exactly, argval must
            #   also start with @ and is therefore an exact match as above.
            # * Wildcards match everything:
            if ruleval == '*':
                return True
            # UID ranges - must be an integer argument:
            if (not ruleval.startswith('@')) and ':' in ruleval and \
             isinstance(argval, int):
                low, high = ruleval.split(':')
                # :max matches exactly
                if not low and int(high) == argval:
                    return True
                # min: matches everything not less than min
                if not high and int(low) <= argval:
                    return True
                # else match in range
                if low and high and (int(low) <= argval <= int(high)):
                    return True
            # GID ranges - argval must be a string starting with '@' and
            # all numbers.
            if ruleval.startswith('@') and ':' in ruleval[1:] and \
             isinstance(argval, str) and \
             argval.startswith('@') and argval[1:].isdigit():
                low, high = ruleval[1:].split(':')
                gid = int(argval[1:])
                # :max matches exactly
                if not low and int(high) == gid:
                    return True
                # min: matches everything not less than min
                if not high and int(low) <= gid:
                    return True
                # else match in range
                if low and high and (int(low) <= gid <= int(high)):
                    return True
            # * We ignore % maxlogin limits here.

        # From the man page: "Note, if you specify a type of '-' but neglect
        # to supply the item and value fields then the module will never
        # enforce any limits on the specified user/group etc."
        if param == 'type' and ruleval == '-':
            return True

        return False

    def find_all(self, **kwargs):
        """
        Find all the rules that match the given parameters.

        The three parameters that can be searched for are 'domain', 'type'
        and 'item'.  These are used as argument names in the keyword argument
        list.  If no parameters are given, no matches are returned.
        """
        matched = []
        # Only work with the list of parameters that are actually specified -
        # this allows us to give other parameters in the future to control
        # the search.
        search_params = [p for p in ['domain', 'type', 'item'] if p in kwargs]
        # If we didn't get a valid list of parameters, then don't do any work
        if not search_params:
            return matched

        for rule in self.rules:
            if all([
               self._matches(param, rule[param], kwargs[param])
               for param in search_params
               ]):
                matched.append(rule)

        return matched



"""
MokutilSbstate - command ``mokutil --sb-state``
===============================================
"""

from insights.core import CommandParser
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.mokutil_sbstate)
class MokutilSbstate(CommandParser):
    """
    Class for parsing the ``mokutil --sb-state`` command.

    Attributes:
        secureboot_enabled (bool): True if SecureBoot is enabled,
                                   False if SecureBoot is disabled, otherwise None.

    Sample output of this command is::

        SecureBoot enabled

    Examples:

        >>> type(mokutil)
        <class 'insights.parsers.mokutil_sbstate.MokutilSbstate'>
        >>> mokutil.secureboot_enabled
        True
    """

    def parse_content(self, content):
        self.secureboot_enabled = None

        non_empty_lines = [line for line in content if line]  # get rid of blank lines
        if "SecureBoot enabled" in non_empty_lines[0]:
            self.secureboot_enabled = True
        elif "SecureBoot disabled" in non_empty_lines[0]:
            self.secureboot_enabled = False



import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers.crypto_policies import CryptoPoliciesStateCurrent
from insights.tests import context_wrap

CONFIG = """
DEFAULT
""".strip()


def test_crypto_policies_state_current():
    result = CryptoPoliciesStateCurrent(context_wrap(CONFIG))
    assert result.value == "DEFAULT"


def test_crypto_policies_state_current_empty():
    with pytest.raises(SkipComponent):
        CryptoPoliciesStateCurrent(context_wrap(""))



"""
PmLogSummary - Command ``pmlogsummary``
=======================================
"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


def parse(data):
    """
    Parse a set of key/value pairs into a hierarchical dictionary of
    typed values

    Arguments:
        data (dict): Input dictionary of key/value pairs

    Returns:
        dict: Hierarchical dictionary with keys separated at "." and type
        conversion of the numerical values
    """
    result = {}

    def typed(x):
        try:
            return float(x)
        except Exception:
            return x

    def insert(k, v):
        cur = result
        key_parts = k.split(".")
        # process the '["xxx"]' part as a sub-key
        if v.startswith('['):
            mk, _, v = v.partition(']')
            key_parts.append(mk.strip('"[]'))
            v = v.strip()

        # walk down the structure to the correct leaf
        for part in key_parts:
            if part not in cur:
                cur[part] = {}
            cur = cur[part]

        # break the value apart and store it
        l, r = v.split(None, 1)
        cur["val"] = typed(l)
        cur["units"] = r.strip()

    def kvs():
        # deal with whitespace and high level splitting
        for line in data:
            line = line.strip()
            if line:
                yield line.split(None, 1)

    for k, v in kvs():
        insert(k, v)

    return result


@parser(Specs.pmlog_summary)
class PmLogSummary(CommandParser, dict):
    """
    Parser to parse the output of the ``pmlogsummary`` command

    Sample output of the command is::

        mem.util.used  3133919.812 Kbyte
        mem.physmem  3997600.000 Kbyte
        kernel.all.cpu.user  0.003 none
        kernel.all.cpu.sys  0.004 none
        kernel.all.cpu.nice  0.000 none
        kernel.all.cpu.steal  0.000 none
        kernel.all.cpu.idle  3.986 none
        kernel.all.pressure.io.full.avg ["10 second"] 0.001 none
        kernel.all.pressure.cpu.some.avg ["1 minute"] 14.942 none
        kernel.all.pressure.memory.full.avg ["5 minute"] 0.002 none
        disk.all.total  0.252 count / sec
        disk.dev.total ["vda"] 0.016 count / sec
        disk.dev.total ["vdb"] 0.445 count / sec
        disk.dev.total ["vdc"] 2.339 count / sec

    Output is parsed and stored as a dictionary.  Each value is
    stored as a dict in the form ``{'val': number or string, 'units': string}``.
    Keys are a hierarchy of the input key value split on the "." character.

    For instance::

        1. Input line "mem.util.used  3133919.812 Kbyte" is parsed as:
            {
                'mem': {
                    'util': {
                        'used': {
                            'val': 3133919.812,
                            'units': 'Kbyte'
                        }
                    }
                }
            }
        2. Input line "disk.dev.total ["vdc"] 2.339 count / sec" is parsed as:
            {
                'disk': {
                    'dev': {
                        'total': {
                            'vdc': {
                                'val': 2.339
                                'units': 'count / sec'
                            }
                        }
                    }
                }
            }

    Example:
        >>> type(pmlog_summary)
        <class 'insights.parsers.pmlog_summary.PmLogSummary'>
        >>> 'mem' in pmlog_summary
        True
        >>> pmlog_summary['disk']['all']['total'] == {'val': 0.252, 'units': 'count / sec'}
        True
        >>> pmlog_summary['disk']['dev']['total']['vdc'] == {'val': 2.339, 'units': 'count / sec'}
        True
        >>> pmlog_summary['kernel']['all']['pressure']['memory']['full']['avg']['5 minute'] == {'val': 0.002, 'units': 'none'}
        True
    """

    def parse_content(self, content):
        data = parse(content)

        if len(data) == 0:
            raise SkipComponent()

        self.update(data)



"""
Combiner for edge computing systems
===================================
This combiner uses the following parsers to determine if the system is an edge computing systems.

* :py:class:`insights.parsers.installed_rpms.InstalledRpms`
* :py:class:`insights.parsers.cmdline.CmdLine`
* :py:class:`insights.parsers.systemd.unitfiles.ListUnits`
* :py:class:`insights.parsers.redhat_release.RedhatRelease`
"""
from insights.core.exceptions import SkipComponent
from insights.core.plugins import combiner
from insights.parsers.cmdline import CmdLine
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.systemd.unitfiles import ListUnits
from insights.parsers.redhat_release import RedhatRelease
from insights.parsers.rpm_ostree_status import RpmOstreeStatus


@combiner(ListUnits, optional=[RpmOstreeStatus, InstalledRpms, CmdLine, RedhatRelease])
class RhelForEdge(object):
    """Combiner for checking if the system is an edge computing system. Edge
    computing as well as the Red Hat CoreOS packages are managed via rpm-ostree.
    Use the string "Red Hat Enterprise Linux release" from
    ``/etc/redhat-release`` to determine an edge computing system. The Red Hat
    CoreOS system will have "Red Hat Enterprise Linux CoreOS release" as the
    string.

    .. note::
        RHEL for EDGE is available and supported since RHEL 8.3.

    When an edge computing system (created from online console edge image) is
    configured to use the automated management, the output of ``rhc status`` is
    as below::

            Connection status for <HOST>:
            - Connected to Red Hat Subscription Management
            - The Red Hat connector daemon is active

    The ``rhcd.service`` running on an edge computing system signifies that it
    is configured to use the automated management.

    Attributes:
        is_edge (bool): True when it is an edge computing system
        is_automated (bool): True when the the edge computing system is configured to use automated management

    .. note::
        It is possible to run ``rhcd.service`` on the edge systems created
        from the cockpit edge image. The **is_automated** attribute is only for
        front-end resolution surface. It is used when the edge image is from
        the online console.

    Examples:
        >>> type(rhel_for_edge_obj)
        <class 'insights.combiners.rhel_for_edge.RhelForEdge'>
        >>> rhel_for_edge_obj.is_edge
        True
        >>> rhel_for_edge_obj.is_automated
        True

    """

    def __init__(self, units, rpmostreestatus, rpms, cmdline, redhatrelease):
        self.is_edge = False
        self.is_automated = False
        if rpmostreestatus:
            origin = rpmostreestatus.query.deployments.origin
            origin_check = [item.value.endswith("edge") for item in origin]
            if origin_check and all(origin_check):
                self.is_edge = True
                if units.is_running("rhcd.service"):
                    self.is_automated = True
        elif rpms and cmdline and redhatrelease:
            if ('rpm-ostree' in rpms and 'yum' not in rpms) and \
                    ('ostree' in cmdline) and \
                    ("red hat enterprise linux release" in redhatrelease.raw.lower()):
                self.is_edge = True
                if units.is_running("rhcd.service"):
                    self.is_automated = True
        else:
            raise SkipComponent("Unable to determine if this system is created from an edge image.")



"""
RandomEntropyAvail - File ``/proc/sys/kernel/random/entropy_avail``
===================================================================

Parser for parsing the ``/proc/sys/kernel/random/entropy_avail`` file.
"""

from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.random_entropy_avail)
class RandomEntropyAvail(Parser):
    """
    Class for parsing the ``/proc/sys/kernel/random/entropy_avail`` file.

    Sample input::

        3137

    Attributes:
        avail_entropy (int): the available entropy

    Examples:
        >>> type(random_entropy_obj)
        <class 'insights.parsers.proc_random_entropy_avail.RandomEntropyAvail'>
        >>> random_entropy_obj.avail_entropy
        3137

    Raises:
        SkipComponent: when there is no expected content in the file
    """

    def parse_content(self, content):
        if len(content) == 1 and content[0].isdigit():
            self.avail_entropy = int(content[0])
        if not hasattr(self, 'avail_entropy'):
            raise SkipComponent('No exptected available entropy value found')



from insights.parsers import mdadm
from insights.parsers.mdadm import MDAdmMetadata, MDAdmDetail
from insights.core.exceptions import SkipComponent
from insights.tests import context_wrap

import doctest
import pytest

MDADM_CONTENT = """
/dev/loop0:
Magic : a92b4efc
Version : 1.0
Feature Map : 0x0
Array UUID : 98e098ef:c8662ce2:2ed2aa5f:7f0416a9
Name : 0
Creation Time : Mon Jun 29 02:16:52 2020
Raid Level : raid1
Raid Devices : 2

Avail Dev Size : 16383968 sectors (7.81 GiB 8.39 GB)
Array Size : 1048576 KiB (1024.00 MiB 1073.74 MB)
Used Dev Size : 2097152 sectors (1024.00 MiB 1073.74 MB)
Super Offset : 16383984 sectors
Unused Space : before=0 sectors, after=14286824 sectors
State : clean
Device UUID : 5e249ed9:a9ee800a:c09c963f:363a18d2

Update Time : Mon Jun 29 02:19:56 2020
Bad Block Log : 512 entries available at offset -8 sectors
Checksum : 395066e8 - correct
Events : 60

Device Role : Active device 0
Array State : AA ('A' == active, '.' == missing, 'R' == replacing)
"""  # noqa


def test_mdadm():
    md = MDAdmMetadata(context_wrap(
        MDADM_CONTENT, path='insights_commands/mdadm_-E_.dev.loop0'
    ))

    # Device assertions
    assert md.device == '/dev/loop0'

    # Information assertions
    assert md['Update Time'] == 'Mon Jun 29 02:19:56 2020'
    assert md['Array Size'] == '1048576 KiB (1024.00 MiB 1073.74 MB)'

    with pytest.raises(SkipComponent) as exc:
        MDAdmMetadata(context_wrap(MDADM_CONTENT))
    assert 'Cannot parse device name from path' in str(exc)


MDADM_D_CONTENT_MD0 = """
/dev/md0:
           Version : 1.2
     Creation Time : Wed Mar 24 14:28:18 2021
        Raid Level : raid5
        Array Size : 21877481472 (20.37 TiB 22.40 TB)
     Used Dev Size : 1562677248 (1490.29 GiB 1600.18 GB)
      Raid Devices : 21
     Total Devices : 22
       Persistence : Superblock is persistent

     Intent Bitmap : Internal

       Update Time : Sat May 13 07:21:53 2023
             State : active, reshaping
    Active Devices : 21
   Working Devices : 22
    Failed Devices : 0
     Spare Devices : 1

            Layout : left-symmetric
        Chunk Size : 512K

Consistency Policy : bitmap

    Reshape Status : 28% complete
     Delta Devices : 6, (15->21)

              Name : hostname:0  (local to host hostname)
              UUID : 245e1231:245e1231:245e1231:245e1231
            Events : 372252

    Number   Major   Minor   RaidDevice State
       0       8      225        0      active sync   /dev/sdo1
       1       8      241        1      active sync   /dev/sdp1
       2       8      177        2      active sync   /dev/sdl1
       3       8      209        3      active sync   /dev/sdn1
       4       8      161        4      active sync   /dev/sdk1
       5       8      145        5      active sync   /dev/sdj1
       6       8      193        6      active sync   /dev/sdm1
       7       8      113        7      active sync   /dev/sdh1
       8       8       81        8      active sync   /dev/sdf1
       9       8      129        9      active sync   /dev/sdi1
      10       8       97       10      active sync   /dev/sdg1
      11       8       33       11      active sync   /dev/sdc1
      12       8       49       12      active sync   /dev/sdd1
      13       8       65       13      active sync   /dev/sde1
      15       8       17       14      active sync   /dev/sdb1
      22      65       81       15      active sync   /dev/sdv1
      21      65       65       16      active sync   /dev/sdu1
      20      65       49       17      active sync   /dev/sdt1
      19      65       33       18      active sync   /dev/sds1
      18      65       17       19      active sync   /dev/sdr1
      17      65        1       20      active sync   /dev/sdq1

      16       8        1        -      spare   /dev/sda1

""".strip()

MDADM_D_CONTENT_MD1 = """
/dev/md1:
          Version : 1.2
      Raid Level : raid5
    Total Devices : 4
      Persistence : Superblock is persistent

            State : inactive
  Working Devices : 4

            Name : any:0
            UUID : 245e1231:245e1231:245e1231:245e1231
          Events : 402

  Number   Major   Minor   RaidDevice

      -     259       16        -        /dev/nvme0n1p1
      -     259       11        -        /dev/nvme1n1p1
      -     259        6        -        /dev/nvme2n1p1
      -     259        1        -        /dev/nvme3n1p1
""".strip()

MDADM_D_CONTENT_MD2 = """
/dev/md2:
           Version : 1.2
     Creation Time : Sun Sep  5 23:19:18 2021
        Raid Level : raid1
        Array Size : 7501333824 (7153.83 GiB 7681.37 GB)
     Used Dev Size : 7501333824 (7153.83 GiB 7681.37 GB)
      Raid Devices : 2
     Total Devices : 2
       Persistence : Superblock is persistent

     Intent Bitmap : Internal

       Update Time : Sun Sep 26 22:18:13 2021
             State : clean
    Active Devices : 2
   Working Devices : 2
    Failed Devices : 0
     Spare Devices : 0

Consistency Policy : bitmap

              Name : hostname:2  (local to host hostname)
              UUID : 245e1231:245e1231:245e1231:245e1231
            Events : 1821

    Number   Major   Minor   RaidDevice State
       0     259        1        0      active sync   /dev/nvme2n1
       1     259        0        1      active sync   /dev/nvme3n1
""".strip()

MDADM_D_CONTENT_MULTI_DEVICES = """
/dev/md/d1p2:
           Version : 1.2
     Creation Time : Thu Jun 17 09:03:18 2021
        Raid Level : raid1
        Array Size : 43239552 (41.24 GiB 44.28 GB)
     Used Dev Size : 43239552 (41.24 GiB 44.28 GB)
      Raid Devices : 2
     Total Devices : 2
       Persistence : Superblock is persistent
     Intent Bitmap : Internal

       Update Time : Mon Dec  6 14:25:25 2021
             State : clean
    Active Devices : 2
   Working Devices : 2
    Failed Devices : 0
     Spare Devices : 0

Consistency Policy : bitmap

              Name : dvrhdbi3s01:3  (local to host dvrhdbi3s01)
              UUID : 245e1231:245e1231:245e1231:245e1231
            Events : 251

    Number   Major   Minor   RaidDevice State
       0      94       33        0      active sync   /dev/dasdi1
       1      94       53        1      active sync   /dev/dasdn1
/dev/md21p3:
           Version : 1.2
     Creation Time : Sun Jun 27 19:14:00 2021
        Raid Level : raid1
        Array Size : 43239552 (41.24 GiB 44.28 GB)
     Used Dev Size : 43239552 (41.24 GiB 44.28 GB)
      Raid Devices : 2
     Total Devices : 2
       Persistence : Superblock is persistent

       Update Time : Sun Dec  5 01:34:48 2021
             State : clean
    Active Devices : 2
   Working Devices : 2
    Failed Devices : 0
     Spare Devices : 0

Consistency Policy : resync

              Name : dvrhdbi3s01:6  (local to host dvrhdbi3s01)
              UUID : 245e1231:245e1231:245e1231:245e1232
            Events : 95

    Number   Major   Minor   RaidDevice State
       0      94       29        0      active sync   /dev/dasdh1
       1      94       37        1      active sync   /dev/dasdj1
""".strip()


def test_mdadm_d():
    mds = MDAdmDetail(context_wrap(MDADM_D_CONTENT_MD0, path='insights_commands/mdadm_-D_.dev.md'))
    assert len(mds) == 1
    md = mds[0]
    assert md.device_name == '/dev/md0'
    assert md['device_name'] == '/dev/md0'
    assert md['Update Time'] == 'Sat May 13 07:21:53 2023'
    assert md['Intent Bitmap'] == 'Internal'
    assert md.is_internal_bitmap is True
    assert len(md.device_table) == 22
    assert md.device_table[0] == {'Major': '8',
                                    'Minor': '225',
                                    'Number': '0',
                                    'RaidDevice': '0',
                                    'State': 'active sync   /dev/sdo1'}
    assert md.device_table[-1] == {'Major': '8',
                                    'Minor': '1',
                                    'Number': '16',
                                    'RaidDevice': '-',
                                    'State': 'spare   /dev/sda1'}

    mds = MDAdmDetail(context_wrap(MDADM_D_CONTENT_MD1))
    assert len(mds) == 1
    md = mds[0]
    assert md.device_name == '/dev/md1'
    assert md['State'] == 'inactive'
    assert md['Working Devices'] == '4'
    assert md.is_internal_bitmap is False
    assert len(md.device_table) == 4
    assert md.device_table[0] == {'Major': '259',
                                    'Minor': '16',
                                    'Number': '-',
                                    'RaidDevice': '-        /dev/nvme0n1p1'}

    mds = MDAdmDetail(context_wrap(MDADM_D_CONTENT_MD2))
    assert len(mds) == 1
    md = mds[0]
    assert md.device_name == '/dev/md2'
    assert md['State'] == 'clean'
    assert md['Working Devices'] == '2'
    assert md.is_internal_bitmap is True
    assert len(md.device_table) == 2
    assert md.device_table[0] == {'Major': '259',
                                    'Minor': '1',
                                    'Number': '0',
                                    'RaidDevice': '0',
                                    'State': 'active sync   /dev/nvme2n1'}

    mds = MDAdmDetail(context_wrap(MDADM_D_CONTENT_MULTI_DEVICES))
    assert len(mds) == 2
    assert len(mds.unparsable_device_list) == 0
    md = mds[0]
    assert md.device_name == '/dev/md/d1p2'
    assert len(md) == 21
    assert md['State'] == 'clean'
    assert md['Consistency Policy'] == 'bitmap'
    assert md.is_internal_bitmap is True
    assert len(md.device_table) == 2
    md = mds[1]
    assert md.device_name == '/dev/md21p3'
    assert md['State'] == 'clean'
    assert md['Consistency Policy'] == 'resync'
    assert md.is_internal_bitmap is False
    assert len(md.device_table) == 2


MDADM_D_CONTENT_EMPTY_TABLE = """
/dev/md123:
           Version : 1.2
     Creation Time : Sun Sep  5 23:19:18 2021
        Raid Level : raid1

    Number   Major   Minor   RaidDevice State
""".strip()

MDADM_D_CONTENT_NO_TABLE = """
/dev/md124:
           Version : 1.2
     Creation Time : Sun Sep  5 23:19:18 2021
        Raid Level : raid1

""".strip()

MDADM_D_CONTENT_NO_DETAIL_LIST = """
/dev/md125:
    Number   Major   Minor   RaidDevice State
       0     259        1        0      active sync   /dev/nvme2n1
       1     259        0        1      active sync   /dev/nvme3n1

""".strip()

MDADM_D_CONTENT_EMPTY_DATA_1 = """
/dev/md126:

    Number   Major   Minor   RaidDevice State

""".strip()

MDADM_D_CONTENT_EMPTY_DATA_2 = """
/dev/md127:

""".strip()

MDADM_D_CONTENT_NO_MD_DEVICES = """
mdadm: cannot open /dev/md*: No such file or directory
""".strip()


def test_mdadm_d_special_cases():
    mds = MDAdmDetail(context_wrap('\n'.join([
                        MDADM_D_CONTENT_EMPTY_TABLE,
                        MDADM_D_CONTENT_EMPTY_DATA_1,
                        MDADM_D_CONTENT_NO_TABLE,
                        MDADM_D_CONTENT_EMPTY_DATA_2,
                        MDADM_D_CONTENT_NO_DETAIL_LIST])))

    md = mds[0]
    assert md.device_name == '/dev/md123'
    assert len(md) == 5
    assert md['Raid Level'] == 'raid1'
    assert md.device_table == []
    assert len(md.device_table) == 0

    md = mds[1]
    assert md.device_name == '/dev/md124'
    assert len(md) == 5
    assert md['Raid Level'] == 'raid1'
    assert len(md.device_table) == 0

    md = mds[2]
    assert md.device_name == '/dev/md125'
    assert len(md) == 2
    assert 'Raid Level' not in md
    assert len(md.device_table) == 2

    assert len(mds.unparsable_device_list) == 2
    mds.unparsable_device_list == ['/dev/md126', '/dev/md127']


def test_mdadm_d_exceptions():
    with pytest.raises(SkipComponent) as exc:
        MDAdmDetail(context_wrap(""))
    assert 'Empty content of command output' in str(exc)

    with pytest.raises(SkipComponent) as exc:
        MDAdmDetail(context_wrap("\n"))
    assert 'Empty content of command output' in str(exc)

    with pytest.raises(SkipComponent) as exc:
        MDAdmDetail(context_wrap('\n'.join([
                      MDADM_D_CONTENT_EMPTY_DATA_1,
                      MDADM_D_CONTENT_EMPTY_DATA_2])))
    assert 'Empty parsed device' in str(exc)


def test_doc_examples():
    env = {
        'mdadm': MDAdmMetadata(context_wrap(
            MDADM_CONTENT, path='insights_commands/mdadm_-E_.dev.loop0'
        )),
        'mdadm_d': MDAdmDetail(context_wrap('\n'.join([MDADM_D_CONTENT_MD2, MDADM_D_CONTENT_MD1])))
    }
    failed, total = doctest.testmod(mdadm, globs=env)
    assert failed == 0



from insights.parsers.systemd import config

SystemdDocker = config.SystemdDocker
SystemdLogindConf = config.SystemdLogindConf
SystemdOpenshiftNode = config.SystemdOpenshiftNode
SystemdSystemConf = config.SystemdSystemConf



"""
KTimerLockless - file ``/sys/kernel/ktimer_lockless_check``
===========================================================
"""
from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.ktimer_lockless)
class KTimerLockless(Parser):
    """
    The KTimerLockless class parses the file ``/sys/kernel/ktimer_lockless_check``.

    Typical output of ``/sys/kernel/ktimer_lockless_check`` file:

        0

    Examples:
        >>> ktimer_lockless.ktimer_lockless_val
        0

    Raises:
        SkipComponent: when input is empty.
    """
    def parse_content(self, content):
        if len(content) == 1 and content[0].isdigit():
            self.ktimer_lockless_val = int(content[0])
        else:
            raise SkipComponent('The file is empty')



"""
Bootc - Command ``bootc``
=========================
"""
from insights.core import JSONParser
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.bootc_status)
class BootcStatus(JSONParser):
    """
    Parses the output of command ``bootc status --json``

    Typical output of the command::

        {
            "apiVersion":"org.containers.bootc/v1alpha1",
            "kind":"BootcHost",
            "metadata":{
                "name":"host"
            },
            "spec":{
                "image":{
                    "image":"192.168.124.1:5000/bootc-insights:latest",
                    "transport":"registry"
                }
            },
            "status":{
                "staged":null,
                "booted":{
                    "image":{
                        "image":{
                            "image":"192.168.124.1:5000/bootc-insights:latest",
                            "transport":"registry"
                        },
                        "version":"stream9.20231213.0",
                        "timestamp":null,
                    },
                    "incompatible":false,
                    "pinned":false,
                    "ostree":{
                        "deploySerial":0
                    }
                },
            }
        }

    Examples:
        >>> type(bootc_status)
        <class 'insights.parsers.bootc.BootcStatus'>
        >>> bootc_status['status']['booted']['image']['version'] == 'stream9.20231213.0'
        True
    """
    pass
